# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.7
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info
if version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_bzlib', [dirname(__file__)])
        except ImportError:
            import _bzlib
            return _bzlib
        if fp is not None:
            try:
                _mod = imp.load_module('_bzlib', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _bzlib = swig_import_helper()
    del swig_import_helper
else:
    import _bzlib
del version_info
try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.


def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr_nondynamic(self, class_type, name, static=1):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    if (not static):
        return object.__getattr__(self, name)
    else:
        raise AttributeError(name)

def _swig_getattr(self, class_type, name):
    return _swig_getattr_nondynamic(self, class_type, name, 0)


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object:
        pass
    _newclass = 0


class State(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, State, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, State, name)
    __repr__ = _swig_repr
    __swig_setmethods__["next_in"] = _bzlib.State_next_in_set
    __swig_getmethods__["next_in"] = _bzlib.State_next_in_get
    if _newclass:
        next_in = _swig_property(_bzlib.State_next_in_get, _bzlib.State_next_in_set)
    __swig_setmethods__["avail_in"] = _bzlib.State_avail_in_set
    __swig_getmethods__["avail_in"] = _bzlib.State_avail_in_get
    if _newclass:
        avail_in = _swig_property(_bzlib.State_avail_in_get, _bzlib.State_avail_in_set)
    __swig_setmethods__["next_out"] = _bzlib.State_next_out_set
    __swig_getmethods__["next_out"] = _bzlib.State_next_out_get
    if _newclass:
        next_out = _swig_property(_bzlib.State_next_out_get, _bzlib.State_next_out_set)
    __swig_setmethods__["avail_out"] = _bzlib.State_avail_out_set
    __swig_getmethods__["avail_out"] = _bzlib.State_avail_out_get
    if _newclass:
        avail_out = _swig_property(_bzlib.State_avail_out_get, _bzlib.State_avail_out_set)
    __swig_setmethods__["state"] = _bzlib.State_state_set
    __swig_getmethods__["state"] = _bzlib.State_state_get
    if _newclass:
        state = _swig_property(_bzlib.State_state_get, _bzlib.State_state_set)

    def __init__(self):
        this = _bzlib.new_State()
        try:
            self.this.append(this)
        except:
            self.this = this
    __swig_destroy__ = _bzlib.delete_State
    __del__ = lambda self: None
State_swigregister = _bzlib.State_swigregister
State_swigregister(State)


def BZ2_SetInBuffer(s, data):
    return _bzlib.BZ2_SetInBuffer(s, data)
BZ2_SetInBuffer = _bzlib.BZ2_SetInBuffer

def BZ2_SetOutBuffer(s, data):
    return _bzlib.BZ2_SetOutBuffer(s, data)
BZ2_SetOutBuffer = _bzlib.BZ2_SetOutBuffer

def BZ2_Init():
    return _bzlib.BZ2_Init()
BZ2_Init = _bzlib.BZ2_Init

def BZ2_Free(s):
    return _bzlib.BZ2_Free(s)
BZ2_Free = _bzlib.BZ2_Free

def BZ2_Decompress(s):
    return _bzlib.BZ2_Decompress(s)
BZ2_Decompress = _bzlib.BZ2_Decompress

_OUTBUFSIZE = 0x1000

class BzException(Exception):
  pass

def decompress(data):
  try:
    state = BZ2_Init()
    BZ2_SetInBuffer(state, data)

    outbuf = bytearray(_OUTBUFSIZE)
    out = bytearray()

    while True:
      BZ2_SetOutBuffer(state, outbuf)
      out1 = state.next_out

      err = BZ2_Decompress(state)
      processed = int(state.next_out) - int(out1)
      if err < 0:
        break

      if processed == 0:
        break

      out += outbuf[:processed]
  finally:
    BZ2_Free(state)

  return out

# This file is compatible with both classic and new-style classes.


